@@ -27,6 +27,7 @@ import java.util.List;
 import org.junit.Test;
 import org.opennms.newts.persistence.cassandra.Utils.MeasurementRowsBuilder;
 import org.opennms.newts.persistence.cassandra.Utils.SampleRowsBuilder;
+import org.opennms.newts.api.Context;
 import org.opennms.newts.api.Duration;
 import org.opennms.newts.api.Measurement;
 import org.opennms.newts.api.MetricType;
@@ -50,7 +51,7 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
         }
 
         // Override the shard period to ensure we test query concurrency
-        getRepository().setResourceShard(Duration.seconds(600));
+        m_contextConfigurations.addContextConfig(Context.DEFAULT_CONTEXT, Duration.seconds(600));
 
         getRepository().insert(writeSamples);
 
@@ -98,11 +99,12 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
         writeSamples(testSamples);
 
         Results<Measurement> results = getRepository().select(
+                Context.DEFAULT_CONTEXT,
                 new Resource("localhost"),
                 Optional.of(Timestamp.fromEpochSeconds(900003600)),
                 Optional.of(Timestamp.fromEpochSeconds(900007200)),
                 rDescriptor,
-                Duration.minutes(60));
+                Optional.of(Duration.minutes(60)));
 
         // Validate results
         assertRowsEqual(expected, results.iterator());
@@ -156,11 +158,12 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
         writeSamples(testSamples);
 
         Results<Measurement> results = getRepository().select(
+                Context.DEFAULT_CONTEXT,
                 new Resource("localhost"),
                 Optional.of(Timestamp.fromEpochSeconds(900003600)),
                 Optional.of(Timestamp.fromEpochSeconds(900007200)),
                 rDescriptor,
-                Duration.minutes(60));
+                Optional.of(Duration.minutes(60)));
 
         // Validate results
         assertRowsEqual(expected, results.iterator());
@@ -214,11 +217,12 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
         writeSamples(testSamples);
 
         Results<Measurement> results = getRepository().select(
+                Context.DEFAULT_CONTEXT,
                 new Resource("localhost"),
                 Optional.of(Timestamp.fromEpochSeconds(900003600)),
                 Optional.of(Timestamp.fromEpochSeconds(900007200)),
                 rDescriptor,
-                Duration.minutes(60));
+                Optional.of(Duration.minutes(60)));
 
         // Validate results
         assertRowsEqual(expected, results.iterator());
@@ -272,11 +276,12 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
         writeSamples(testSamples);
 
         Results<Measurement> results = getRepository().select(
+                Context.DEFAULT_CONTEXT,
                 new Resource("localhost"),
                 Optional.of(Timestamp.fromEpochSeconds(900003600)),
                 Optional.of(Timestamp.fromEpochSeconds(900007200)),
                 rDescriptor,
-                Duration.minutes(60));
+                Optional.of(Duration.minutes(60)));
 
         // Validate results
         assertRowsEqual(expected, results.iterator());
@@ -288,4 +293,63 @@ public class InsertSelectMeasurementsITCase extends AbstractCassandraTestCase {
 
     }
 
+    @Test
+    public void testWithDefaultResolution() {
+
+        Iterator<Row<Sample>> testSamples = new SampleRowsBuilder(new Resource("localhost"), MetricType.GAUGE)
+                .row(900000000).element("mGauge", 1)        // Thu Jul  9 11:00:00 CDT 1998
+                .row(900000300).element("mGauge", 1)
+                .row(900000600).element("mGauge", 1)
+                .row(900000900).element("mGauge", 1)
+                .row(900001200).element("mGauge", 1)
+                .row(900001500).element("mGauge", 1)
+                .row(900001800).element("mGauge", 1, mapFor("a", "1"))
+                .row(900002100).element("mGauge", 3)
+                .row(900002400).element("mGauge", 3, mapFor("b", "2"))
+                .row(900002700).element("mGauge", 3)
+                .row(900003000).element("mGauge", 3)
+                .row(900003300).element("mGauge", 3)
+                .row(900003600).element("mGauge", 3)
+                .row(900003900).element("mGauge", 1)
+                .row(900004200).element("mGauge", 1)
+                .row(900004500).element("mGauge", 1)
+                .row(900004800).element("mGauge", 1)
+                .row(900005100).element("mGauge", 1, mapFor("c", "3"))
+                .row(900005400).element("mGauge", 1)
+                .row(900005700).element("mGauge", 3, mapFor("d", "4"))
+                .row(900006000).element("mGauge", 3)
+                .row(900006300).element("mGauge", 3)
+                .row(900006600).element("mGauge", 3)
+                .row(900006900).element("mGauge", 3)
+                .row(900007200).element("mGauge", 3)        // Thu Jul  9 13:00:00 CDT 1998
+                .build();
+
+        ResultDescriptor rDescriptor = new ResultDescriptor(Duration.seconds(300))
+            .datasource("mGauge-avg", "mGauge", Duration.seconds(600), AVERAGE).export("mGauge-avg");
+
+        Iterator<Row<Measurement>> expected = new MeasurementRowsBuilder(new Resource("localhost"))
+                .row(900003600).element("mGauge-avg", 3)
+                .row(900004200).element("mGauge-avg", 1)
+                .row(900004800).element("mGauge-avg", 1)
+                .row(900005400).element("mGauge-avg", 1)
+                .row(900006000).element("mGauge-avg", 3)
+                .row(900006600).element("mGauge-avg", 3)
+                .row(900007200).element("mGauge-avg", 3)
+                .build();
+
+        writeSamples(testSamples);
+
+        Results<Measurement> results = getRepository().select(
+                Context.DEFAULT_CONTEXT,
+                new Resource("localhost"),
+                Optional.of(Timestamp.fromEpochSeconds(900003600)),
+                Optional.of(Timestamp.fromEpochSeconds(900007200)),
+                rDescriptor,
+                Optional.<Duration>absent());
+
+        // Validate results
+        assertRowsEqual(expected, results.iterator());
+
+    }
+
 }
